/*

Sceptre television remote codes, for the Arduino IRLib2 library,
(based on the rawSend sketch from the IRLib2 library.)

The codes were derived from the AnalysIR sketch, with the IR receiver,
(a TSOP38338 was used for the development,) in the middle of a large
desk, about 2" from the remote LED.

Due to the size of the IR code data, at least an Arduino Mega, (an
Elegoo EL-CB-003 MEGA 2560 R3 Board ATmega2560 ATMEGA16U2 was used for
the development,) will be required. (A Genuine Arduino UNO was tested,
also, with a reduced number of buttons represented.)

The TSOP38338 IR receiver output pin, (pin 1 = OUT on the TSOP38338
data sheet,) is connected to the Arduino input pin 2, (both Arduino
MEGA and UNO.)

The IR LED circuit is connected to the Arduino MEGA pin 9~.

The IR LED circuit is connected to the Arduino UNO pin 3~.

This sketch is intended to use the xr2ACM0, or, sprawACM0 raw terminal
program as a command line input program from the computer terminal.

    The Sony remote protocol is based on a base timing value of 600
    uS, and the timing values for a record are:

        1) A Mark of 4 X 600 uS = 2.4 mS

        2) Data Logical '0' a Space of 1 * 600 uS followed
           by a Mark of 1 * 600 uS

        3) Data Logical '1' a Space of 1 * 600 uS followed
           by a Mark of 2 * 600 uS

        4) A Record interval is always 75 * 600 uS = 45 mS

        5) A Record is sent three times

        6) A Space of at least 10 mS is sent at the end of a
           completion of the three Records being sent

    This simplifies the changing of the Sony timing values derived
    from the IRLib2 library, AnalysIR sketch, which can be rounded to
    the closest 600 uS value.

    The sony.timing(1) program was used to alter the Sceptre
    television remote timing values derived from the Arduino AnalysIR
    sketch, to industry standard values.

    The Sony (SIRC,) IR carrier frequency is 40 kHz.

    The NEC remote protocol is based on a base timing value of 562.5
    uS, and the timing values for a record are:

        1) A Mark of 16 X 562.5 uS = 9 mS

        2) A Space of 8 X 562.5 uS = 4.5 mS

        3) Data Marks of 1 X 562.5 uS = 0.5625 mS

        4) Data Spaces of 1 X 562.5 uS = 0.5625 mS

        5) An EOR Space of 71 * 562.5 uS = 39.9375 mS between repeat codes

        The repeat code is:

        1) A Mark of 16 X 562.5 uS = 9 mS

        2) A Space of 4 X 562.5 uS = 2.25 mS

        3) A Mark of 1 X 562.5 uS = 0.5625 mS

    This simplifies the changing of the NEC timing values derived from
    the IRLib2 library, AnalysIR sketch, which can be rounded to the
    closest 562.5 uS value.

    The nec.timing(1) program was used to alter the Dingsun remote
    timing values derived from the Arduino AnalysIR sketch, to
    industry standard values.

    The NEC IR carrier frequency is 38 kHz.

    The data structure is too large for an Arduino Uno, but note that
    the first unsigned integer button's frame is repeated three times,
    reducing the Sony data structure by about a factor of 3.

*/

#include <IRLibSendBase.h> // We need the base code
#include <IRLib_HashRaw.h> // Only use raw sender

IRsendRaw mySender;

void setup ()
{
  Serial.begin (9600);
  delay (2000); while (!Serial); //delay for Leonardo
  Serial.print ("Ready: ");
}

// '+' Dingsun IR Remote Volume Control, remote Volume Up button, NEC
// protocol, 75 fields,
unsigned int NecVolumeUp[] = \
{9000, 4500, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 1688, 563, 1688, 563, 1688, 563, 1688, 563, \
 1688, 563, 1688, 563, 1688, 563, 1688, 563, 563, 563, 563, 563, 1688, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, \
 563, 1688, 563, 563, 563, 1688, 563, 1688, 563, 1688, 563, 1688, 563, 1688, 563, 1688, 563, 39938, 9000, 2250, 563, 31500, 9000, 2250, 563 };

// '-' Dingsun IR Remote Volume Control, remote Volume Down button,
// NEC protocol, 75 fields,
unsigned int NecVolumeDown[] = \
{9000, 4500, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 1688, 563, 1688, 563, 1688, 563, 1688, 563, \
 1688, 563, 1688, 563, 1688, 563, 1688, 563, 563, 563, 563, 563, 563, 563, 563, 563, 1688, 563, 563, 563, 563, 563, 563, 563, 563, \
 563, 1688, 563, 1688, 563, 1688, 563, 563, 563, 1688, 563, 1688, 563, 1688, 563, 1688, 563, 39938, 9000, 2250, 563, 31500, 9000, 2250, 563 };

// 'Mute' Dingsun Mute IR Remote Volume Control, remote Mute button,
// NEC protocol 75 fields,
unsigned int NecMute[] = \
{9000, 4500, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 1688, 563, 1688, 563, 1688, 563, 1688, 563, \
 1688, 563, 1688, 563, 1688, 563, 1688, 563, 563, 563, 563, 563, 1688, 563, 563, 563, 1688, 563, 1688, 563, 563, 563, 563, 563, 563, \
 563, 1688, 563, 563, 563, 1688, 563, 563, 563, 563, 563, 1688, 563, 1688, 563, 1688, 563, 39938, 9000, 2250, 563, 31500, 9000, 2250, 563 };

// '0' Sceptre TV remote 0 button, Sony (SIRC,) protocol, 26
// fields, sent (at least 3,) times,
unsigned int SonyButton0[] = \
{2400, 600, 1200, 600, 600, 600, 600, 600, 1200, 600, 600, 600, 600, 600, 600, 600, 1200, 600, 600, 600, 600, 600, 600, 600, 600, 27000, \
 2400, 600, 1200, 600, 600, 600, 600, 600, 1200, 600, 600, 600, 600, 600, 600, 600, 1200, 600, 600, 600, 600, 600, 600, 600, 600, 27000, \
 2400, 600, 1200, 600, 600, 600, 600, 600, 1200, 600, 600, 600, 600, 600, 600, 600, 1200, 600, 600, 600, 600, 600, 600, 600, 600 };

// '1' Sceptre TV remote 1 button, Sony (SIRC,) protocol, 26
// fields, sent (at least 3,) times,
unsigned int SonyButton1[] = \
{2400, 600, 600, 600, 600, 600, 600, 600, 600, 600, 600, 600, 600, 600, 600, 600, 1200, 600, 600, 600, 600, 600, 600, 600, 600, 28200, \
 2400, 600, 600, 600, 600, 600, 600, 600, 600, 600, 600, 600, 600, 600, 600, 600, 1200, 600, 600, 600, 600, 600, 600, 600, 600, 28200, \
 2400, 600, 600, 600, 600, 600, 600, 600, 600, 600, 600, 600, 600, 600, 600, 600, 1200, 600, 600, 600, 600, 600, 600, 600, 600 };

// '2' Sceptre TV remote 2 button, Sony (SIRC,) protocol, 26
// fields, sent (at least 3,) times,
unsigned int SonyButton2[] = \
{2400, 600, 600, 600, 1200, 600, 600, 600, 600, 600, 600, 600, 600, 600, 600, 600, 1200, 600, 600, 600, 600, 600, 600, 600, 600, 27000, \
 2400, 600, 600, 600, 1200, 600, 600, 600, 600, 600, 600, 600, 600, 600, 600, 600, 1200, 600, 600, 600, 600, 600, 600, 600, 600, 27000, \
 2400, 600, 600, 600, 1200, 600, 600, 600, 600, 600, 600, 600, 600, 600, 600, 600, 1200, 600, 600, 600, 600, 600, 600, 600, 600 };

// '3' Sceptre TV remote 3 button, Sony (SIRC,) protocol, 26
// fields, sent (at least 3,) times,
unsigned int SonyButton3[] = \
{2400, 600, 600, 600, 1200, 600, 600, 600, 600, 600, 600, 600, 600, 600, 600, 600, 1200, 600, 600, 600, 600, 600, 600, 600, 600, 27600, \
 2400, 600, 600, 600, 1200, 600, 600, 600, 600, 600, 600, 600, 600, 600, 600, 600, 1200, 600, 600, 600, 600, 600, 600, 600, 600, 27600, \
 2400, 600, 600, 600, 1200, 600, 600, 600, 600, 600, 600, 600, 600, 600, 600, 600, 1200, 600, 600, 600, 600, 600, 600, 600, 600 };

// '4' Sceptre TV remote 4 button, Sony (SIRC,) protocol, 26
// fields, sent (at least 3,) times,
unsigned int SonyButton4[] = \
{2400, 600, 1200, 600, 1200, 600, 600, 600, 600, 600, 600, 600, 600, 600, 600, 600, 1200, 600, 600, 600, 600, 600, 600, 600, 600, 27000, \
 2400, 600, 1200, 600, 1200, 600, 600, 600, 600, 600, 600, 600, 600, 600, 600, 600, 1200, 600, 600, 600, 600, 600, 600, 600, 600, 27000, \
 2400, 600, 1200, 600, 1200, 600, 600, 600, 600, 600, 600, 600, 600, 600, 600, 600, 1200, 600, 600, 600, 600, 600, 600, 600, 600 };

// '5' Sceptre TV remote 5 button, Sony (SIRC,) protocol, 26
// fields, sent (at least 3,) times,
unsigned int SonyButton5[] = \
{2400, 600, 600, 600, 600, 600, 1200, 600, 600, 600, 600, 600, 600, 600, 600, 600, 1200, 600, 600, 600, 600, 600, 600, 600, 600, 27600, \
 2400, 600, 600, 600, 600, 600, 1200, 600, 600, 600, 600, 600, 600, 600, 600, 600, 1200, 600, 600, 600, 600, 600, 600, 600, 600, 27600, \
 2400, 600, 600, 600, 600, 600, 1200, 600, 600, 600, 600, 600, 600, 600, 600, 600, 1200, 600, 600, 600, 600, 600, 600, 600, 600 };

// '6' Sceptre TV remote 6 button, Sony (SIRC,) protocol, 26
// fields, sent (at least 3,) times,
unsigned int SonyButton6[] = \
{2400, 600, 1200, 600, 600, 600, 1200, 600, 600, 600, 600, 600, 600, 600, 600, 600, 1200, 600, 600, 600, 600, 600, 600, 600, 600, 26400, \
 2400, 600, 1200, 600, 600, 600, 1200, 600, 600, 600, 600, 600, 600, 600, 600, 600, 1200, 600, 600, 600, 600, 600, 600, 600, 600, 26400, \
 2400, 600, 1200, 600, 600, 600, 1200, 600, 600, 600, 600, 600, 600, 600, 600, 600, 1200, 600, 600, 600, 600, 600, 600, 600, 600 };

// '7' Sceptre TV remote 7 button, Sony (SIRC,) protocol, 26
// fields, sent (at least 3,) times,
unsigned int SonyButton7[] = \
{2400, 600, 600, 600, 1200, 600, 1200, 600, 600, 600, 600, 600, 600, 600, 600, 600, 1200, 600, 600, 600, 600, 600, 600, 600, 600, 27000, \
 2400, 600, 600, 600, 1200, 600, 1200, 600, 600, 600, 600, 600, 600, 600, 600, 600, 1200, 600, 600, 600, 600, 600, 600, 600, 600, 27000, \
 2400, 600, 600, 600, 1200, 600, 1200, 600, 600, 600, 600, 600, 600, 600, 600, 600, 1200, 600, 600, 600, 600, 600, 600, 600, 600 };

// '8' Sceptre TV remote 8 button, Sony (SIRC,) protocol, 26
// fields, sent (at least 3,) times,
unsigned int SonyButton8[] = \
{2400, 600, 1200, 600, 1200, 600, 1200, 600, 600, 600, 600, 600, 600, 600, 600, 600, 1200, 600, 600, 600, 600, 600, 600, 600, 600, 26400, \
 2400, 600, 1200, 600, 1200, 600, 1200, 600, 600, 600, 600, 600, 600, 600, 600, 600, 1200, 600, 600, 600, 600, 600, 600, 600, 600, 26400, \
 2400, 600, 1200, 600, 1200, 600, 1200, 600, 600, 600, 600, 600, 600, 600, 600, 600, 1200, 600, 600, 600, 600, 600, 600, 600, 600 };

// '9' Sceptre TV remote 9 button, Sony (SIRC,) protocol, 26
// fields, sent (at least 3,) times,
unsigned int SonyButton9[] = \
{2400, 600, 600, 600, 600, 600, 600, 600, 1200, 600, 600, 600, 600, 600, 600, 600, 1200, 600, 600, 600, 600, 600, 600, 600, 600, 27600, \
 2400, 600, 600, 600, 600, 600, 600, 600, 1200, 600, 600, 600, 600, 600, 600, 600, 1200, 600, 600, 600, 600, 600, 600, 600, 600, 27600, \
 2400, 600, 600, 600, 600, 600, 600, 600, 1200, 600, 600, 600, 600, 600, 600, 600, 1200, 600, 600, 600, 600, 600, 600, 600, 600 };

// 'c' Sceptre TV remote CH- button, Sony (SIRC,) protocol, 26
// fields, sent (at least 3,) times,
unsigned int SonyButtonChDown[] = \
{2400, 600, 1200, 600, 600, 600, 600, 600, 600, 600, 1200, 600, 600, 600, 600, 600, 1200, 600, 600, 600, 600, 600, 600, 600, 600, 27000, \
 2400, 600, 1200, 600, 600, 600, 600, 600, 600, 600, 1200, 600, 600, 600, 600, 600, 1200, 600, 600, 600, 600, 600, 600, 600, 600, 27000, \
 2400, 600, 1200, 600, 600, 600, 600, 600, 600, 600, 1200, 600, 600, 600, 600, 600, 1200, 600, 600, 600, 600, 600, 600, 600, 600 };

// 'C' Sceptre TV remote CH+ button, Sony (SIRC,) protocol, 26
// fields, sent (at least 3,) times,
unsigned int SonyButtonChUp[] = \
{2400, 600, 600, 600, 600, 600, 600, 600, 600, 600, 1200, 600, 600, 600, 600, 600, 1200, 600, 600, 600, 600, 600, 600, 600, 600, 27600, \
 2400, 600, 600, 600, 600, 600, 600, 600, 600, 600, 1200, 600, 600, 600, 600, 600, 1200, 600, 600, 600, 600, 600, 600, 600, 600, 27600, \
 2400, 600, 600, 600, 600, 600, 600, 600, 600, 600, 1200, 600, 600, 600, 600, 600, 1200, 600, 600, 600, 600, 600, 600, 600, 600 };

// 'd' Sceptre TV remote Down button, Sony (SIRC,) protocol, 26
// fields, sent (at least 3,) times,
unsigned int SonyButtonDown[] = \
{2400, 600, 1200, 600, 600, 600, 1200, 600, 600, 600, 1200, 600, 1200, 600, 1200, 600, 1200, 600, 600, 600, 600, 600, 600, 600, 600, 25200, \
 2400, 600, 1200, 600, 600, 600, 1200, 600, 600, 600, 1200, 600, 1200, 600, 1200, 600, 1200, 600, 600, 600, 600, 600, 600, 600, 600, 25200, \
 2400, 600, 1200, 600, 600, 600, 1200, 600, 600, 600, 1200, 600, 1200, 600, 1200, 600, 1200, 600, 600, 600, 600, 600, 600, 600, 600 };

// 'e' Sceptre TV remote ENTER button, Sony (SIRC,) protocol, 26
// fields, sent (at least 3,) times,
unsigned int SonyButtonEnter[] = \
{2400, 600, 1200, 600, 1200, 600, 600, 600, 1200, 600, 600, 600, 600, 600, 600, 600, 1200, 600, 600, 600, 600, 600, 600, 600, 600, 26400, \
 2400, 600, 1200, 600, 1200, 600, 600, 600, 1200, 600, 600, 600, 600, 600, 600, 600, 1200, 600, 600, 600, 600, 600, 600, 600, 600, 26400, \
 2400, 600, 1200, 600, 1200, 600, 600, 600, 1200, 600, 600, 600, 600, 600, 600, 600, 1200, 600, 600, 600, 600, 600, 600, 600, 600 };

// 'E' Sceptre TV remote Exit button, Sony (SIRC,) protocol, 26
// fields, sent (at least 3,) times,
unsigned int SonyButtonExit[] = \
{2400, 600, 600, 600, 1200, 600, 1200, 600, 600, 600, 1200, 600, 600, 600, 600, 600, 1200, 600, 600, 600, 600, 600, 600, 600, 600, 26400, \
 2400, 600, 600, 600, 1200, 600, 1200, 600, 600, 600, 1200, 600, 600, 600, 600, 600, 1200, 600, 600, 600, 600, 600, 600, 600, 600, 26400, \
 2400, 600, 600, 600, 1200, 600, 1200, 600, 600, 600, 1200, 600, 600, 600, 600, 600, 1200, 600, 600, 600, 600, 600, 600, 600, 600 };

// 'I' Sceptre TV remote Guide button, Sony (SIRC,) protocol, 26
// fields, sent (at least 3,) times,
unsigned int SonyButtonGuide[] = \
{2400, 600, 1200, 600, 600, 600, 1200, 600, 600, 600, 600, 600, 1200, 600, 1200, 600, 1200, 600, 600, 600, 600, 600, 600, 600, 600, 25800, \
 2400, 600, 1200, 600, 600, 600, 1200, 600, 600, 600, 600, 600, 1200, 600, 1200, 600, 1200, 600, 600, 600, 600, 600, 600, 600, 600, 25800, \
 2400, 600, 1200, 600, 600, 600, 1200, 600, 600, 600, 600, 600, 1200, 600, 1200, 600, 1200, 600, 600, 600, 600, 600, 600, 600, 600 };

// 'l' Sceptre TV remote Left button, Sony (SIRC,) protocol, 26
// fields, sent (at least 3,) times,
unsigned int SonyButtonLeft[] = \
{2400, 600, 600, 600, 600, 600, 1200, 600, 600, 600, 1200, 600, 1200, 600, 600, 600, 1200, 600, 600, 600, 600, 600, 600, 600, 600, 26400, \
 2400, 600, 600, 600, 600, 600, 1200, 600, 600, 600, 1200, 600, 1200, 600, 600, 600, 1200, 600, 600, 600, 600, 600, 600, 600, 600, 26400, \
 2400, 600, 600, 600, 600, 600, 1200, 600, 600, 600, 1200, 600, 1200, 600, 600, 600, 1200, 600, 600, 600, 600, 600, 600, 600, 600 };

// 'M' Sceptre TV remote Menu button, Sony (SIRC,) protocol, 26
// fields, sent (at least 3,) times,
unsigned int SonyButtonMenu[] = \
{2400, 600, 600, 600, 600, 600, 600, 600, 600, 600, 600, 600, 1200, 600, 1200, 600, 1200, 600, 600, 600, 600, 600, 600, 600, 600, 27000, \
 2400, 600, 600, 600, 600, 600, 600, 600, 600, 600, 600, 600, 1200, 600, 1200, 600, 1200, 600, 600, 600, 600, 600, 600, 600, 600, 27000, \
 2400, 600, 600, 600, 600, 600, 600, 600, 600, 600, 600, 600, 1200, 600, 1200, 600, 1200, 600, 600, 600, 600, 600, 600, 600, 600 };

// 'o' Sceptre TV remote Power button, Sony (SIRC,) protocol, 26
// fields, sent (at least 3,) times,
unsigned int SonyButtonPower[] = \
{2400, 600, 1200, 600, 600, 600, 1200, 600, 600, 600, 1200, 600, 600, 600, 600, 600, 1200, 600, 600, 600, 600, 600, 600, 600, 600, 25800, \
 2400, 600, 1200, 600, 600, 600, 1200, 600, 600, 600, 1200, 600, 600, 600, 600, 600, 1200, 600, 600, 600, 600, 600, 600, 600, 600, 25800, \
 2400, 600, 1200, 600, 600, 600, 1200, 600, 600, 600, 1200, 600, 600, 600, 600, 600, 1200, 600, 600, 600, 600, 600, 600, 600, 600 };

// 'r' Sceptre TV remote Right button, Sony (SIRC,) protocol, 26
// fields, sent (at least 3,) times,
unsigned int SonyButtonRight[] = \
{2400, 600, 1200, 600, 1200, 600, 600, 600, 600, 600, 1200, 600, 1200, 600, 600, 600, 1200, 600, 600, 600, 600, 600, 600, 600, 600, 25800, \
 2400, 600, 1200, 600, 1200, 600, 600, 600, 600, 600, 1200, 600, 1200, 600, 600, 600, 1200, 600, 600, 600, 600, 600, 600, 600, 600, 25800, \
 2400, 600, 1200, 600, 1200, 600, 600, 600, 600, 600, 1200, 600, 1200, 600, 600, 600, 1200, 600, 600, 600, 600, 600, 600, 600, 600 };

// 's' Sceptre TV remote Source button, Sony (SIRC,) protocol, 26
// fields, sent (at least 3,) times,
unsigned int SonyButtonSource[] = \
{2400, 600, 1200, 600, 600, 600, 1200, 600, 600, 600, 600, 600, 1200, 600, 600, 600, 1200, 600, 600, 600, 600, 600, 600, 600, 600, 26400, \
 2400, 600, 1200, 600, 600, 600, 1200, 600, 600, 600, 600, 600, 1200, 600, 600, 600, 1200, 600, 600, 600, 600, 600, 600, 600, 600, 26400, \
 2400, 600, 1200, 600, 600, 600, 1200, 600, 600, 600, 600, 600, 1200, 600, 600, 600, 1200, 600, 600, 600, 600, 600, 600, 600, 600 };

// 'u' Sceptre TV remote Up button, Sony (SIRC,) protocol, 26
// fields, sent (at least 3,) times,
unsigned int SonyButtonUp[] = \
{2400, 600, 600, 600, 600, 600, 1200, 600, 600, 600, 1200, 600, 1200, 600, 1200, 600, 1200, 600, 600, 600, 600, 600, 600, 600, 600, 25800, \
 2400, 600, 600, 600, 600, 600, 1200, 600, 600, 600, 1200, 600, 1200, 600, 1200, 600, 1200, 600, 600, 600, 600, 600, 600, 600, 600, 25800, \
 2400, 600, 600, 600, 600, 600, 1200, 600, 600, 600, 1200, 600, 1200, 600, 1200, 600, 1200, 600, 600, 600, 600, 600, 600, 600, 600 };

// // 'v' Sceptre TV remote VOL- button, Sony (SIRC,) protocol, 26
// // fields, sent (at least 3,) times,
// unsigned int SonyButtonVolDown[] = \
// {2400, 600, 1200, 600, 1200, 600, 600, 600, 600, 600, 1200, 600, 600, 600, 600, 600, 1200, 600, 600, 600, 600, 600, 600, 600, 600, 26400, \
//  2400, 600, 1200, 600, 1200, 600, 600, 600, 600, 600, 1200, 600, 600, 600, 600, 600, 1200, 600, 600, 600, 600, 600, 600, 600, 600, 26400, \
//  2400, 600, 1200, 600, 1200, 600, 600, 600, 600, 600, 1200, 600, 600, 600, 600, 600, 1200, 600, 600, 600, 600, 600, 600, 600, 600 };
//
// // 'V' Sceptre TV remote VOL+ button, Sony (SIRC,) protocol, 26
// // fields, sent (at least 3,) times,
// unsigned int SonyButtonVolUp[] = \
// {2400, 600, 600, 600, 1200, 600, 600, 600, 600, 600, 1200, 600, 600, 600, 600, 600, 1200, 600, 600, 600, 600, 600, 600, 600, 600, 27000, \
//  2400, 600, 600, 600, 1200, 600, 600, 600, 600, 600, 1200, 600, 600, 600, 600, 600, 1200, 600, 600, 600, 600, 600, 600, 600, 600, 27000, \
//  2400, 600, 600, 600, 1200, 600, 600, 600, 600, 600, 1200, 600, 600, 600, 600, 600, 1200, 600, 600, 600, 600, 600, 600, 600, 600 };

// Remote vs. terminal keyboard key mapping:

char help[] = \
    "\n\n"
    "Remote                   Single Character\n"
    "Button                   Keyboard Key\n"
    "\n"
    "Power button, (On/Off)  'O', 'o'\n"
    "0 button                '0'\n"
    "1 button                '1'\n"
    "2 button                '2'\n"
    "3 button                '3'\n"
    "4 button                '4'\n"
    "5 button                '5'\n"
    "6 button                '6'\n"
    "7 button                '7'\n"
    "8 button                '8'\n"
    "9 button                '9'\n"
    "CH+ button              'C', Insert, Ins\n"
    "CH- button              'c', Delete, Del\n"
    "Down Arrow button       'D', Down Arrow\n"
    "Exit button             'E', 'e', End\n"
    "Menu button             'G', 'g', Home\n"
    "(This Help)             'H', 'h', '?'\n"
    "Guide button            'I', 'i'\n"
    "Enter button            'k', 'K', Enter\n"
    "Left Arrow button       'L', Left Arrow\n"
    "Mute button, (Toggle)   'M', 'm'\n"
    "(Quit)                  'Q', 'q',\n"
    "Right Arrow button      'R', Right Arrow\n"
    "Up Arrow button         'U', Up Arrow\n"
    "Volume Up button        'V', '+'\n"
    "Volume Down button      'v', '-'\n"
    "\n";

void loop ()
{
    char ch; // Input character from the serial line

    int sz, // Size of unsigned int array to be sent
        button_error = 0, // No such button error, no error = 0, error = 1
        nec = 0, // nec button, 0 = no, 1 = yes
        sony = 0; // sony button, 0 = no, 1 = yes

    unsigned int *ary; // Reference to unsigned int array to be sent

    if (Serial.available ())
    {
        ch = Serial.read ();
        button_error = 0; // Reset the no such button error
        nec = 0; // reset the nec button, 0 = no, 1 = yes
        sony = 0; // reset the sony button, 0 = no, 1 = yes

        switch (ch)
        {

            case 'O':
            case 'o':

                ary = SonyButtonPower;
                sz = sizeof (SonyButtonPower);
                sony = 1; // set the sony button, 0 = no, 1 = yes
                break;

            case '0':

                ary = SonyButton0;
                sz = sizeof (SonyButton0);
                sony = 1; // set the sony button, 0 = no, 1 = yes
                break;

            case '1':

                ary = SonyButton1;
                sz = sizeof (SonyButton1);
                sony = 1; // set the sony button, 0 = no, 1 = yes
                break;

            case '2':

                ary = SonyButton2;
                sz = sizeof (SonyButton2);
                sony = 1; // set the sony button, 0 = no, 1 = yes
                break;

            case '3':

                ary = SonyButton3;
                sz = sizeof (SonyButton3);
                sony = 1; // set the sony button, 0 = no, 1 = yes
                break;

            case '4':

                ary = SonyButton4;
                sz = sizeof (SonyButton4);
                sony = 1; // set the sony button, 0 = no, 1 = yes
                break;

            case '5':

                ary = SonyButton5;
                sz = sizeof (SonyButton5);
                sony = 1; // set the sony button, 0 = no, 1 = yes
                break;

            case '6':

                ary = SonyButton6;
                sz = sizeof (SonyButton6);
                sony = 1; // set the sony button, 0 = no, 1 = yes
                break;

            case '7':

                ary = SonyButton7;
                sz = sizeof (SonyButton7);
                sony = 1; // set the sony button, 0 = no, 1 = yes
                break;

            case '8':

                ary = SonyButton8;
                sz = sizeof (SonyButton8);
                sony = 1; // set the sony button, 0 = no, 1 = yes
                break;

            case '9':

                ary = SonyButton9;
                sz = sizeof (SonyButton9);
                sony = 1; // set the sony button, 0 = no, 1 = yes
                break;

            case 'C':

                ary = SonyButtonChUp;
                sz = sizeof (SonyButtonChUp);
                sony = 1; // set the sony button, 0 = no, 1 = yes
                break;

            case 'c':

                ary = SonyButtonChDown;
                sz = sizeof (SonyButtonChDown);
                sony = 1; // set the sony button, 0 = no, 1 = yes
                break;

            case 'D':

                ary = SonyButtonDown;
                sz = sizeof (SonyButtonDown);
                sony = 1; // set the sony button, 0 = no, 1 = yes
                break;

            case 'E':
            case 'e':

                ary = SonyButtonExit;
                sz = sizeof (SonyButtonExit);
                sony = 1; // set the sony button, 0 = no, 1 = yes
                break;

            case 'G':
            case 'g':

                ary = SonyButtonMenu;
                sz = sizeof (SonyButtonMenu);
                sony = 1; // set the sony button, 0 = no, 1 = yes
                break;

            case 'h':
            case 'H':
            case '?':

                Serial.print (help);
                button_error = 1;
                break;

            case 'I':
            case 'i':

                ary = SonyButtonGuide;
                sz = sizeof (SonyButtonGuide);
                sony = 1; // set the sony button, 0 = no, 1 = yes
                break;

            case 'K':
            case 'k':
            case '\n':

                ch = 'K'; // '\n' would skip a line, and not usefule for diagnostics
                ary = SonyButtonEnter;
                sz = sizeof (SonyButtonEnter);
                sony = 1; // set the sony button, 0 = no, 1 = yes
                break; // The 'k' array is the OK button and will map to the Enter key

            case 'L':

                ary = SonyButtonLeft;
                sz = sizeof (SonyButtonLeft);
                sony = 1; // set the sony button, 0 = no, 1 = yes
                break;

            case 'M':
            case 'm':

                ary = NecMute;
                sz = sizeof (NecMute);
                nec = 1; // set the nec button, 0 = no, 1 = yes
                break;

            case 'R':

                ary = SonyButtonRight;
                sz = sizeof (SonyButtonRight);
                sony = 1; // set the sony button, 0 = no, 1 = yes
                break;

            case 'S':
            case 's':

                ary = SonyButtonSource;
                sz = sizeof (SonyButtonSource);
                sony = 1; // set the sony button, 0 = no, 1 = yes
                break;

            case 'U':

                ary = SonyButtonUp;
                sz = sizeof (SonyButtonUp);
                sony = 1; // set the sony button, 0 = no, 1 = yes
                break;

            case 'V':
            case '+':

                ary = NecVolumeUp;
                sz = sizeof (NecVolumeUp);
                nec = 1; // set the nec button, 0 = no, 1 = yes
                break;

            case 'v':
            case '-':

                ary = NecVolumeDown;
                sz = sizeof (NecVolumeDown);
                nec = 1; // set the nec button, 0 = no, 1 = yes
                break;

            case '\r':

                button_error = 1; // Set the no such button error, to prevent sending by the mySender.send function
                break; // CR = '\r' is ignored since the serial device may be a Unix terminal program

            default:

                Serial.write (ch); // Print the received character
                Serial.print (", (Ignored);");
                button_error = 1; // Set the no such button error
                break;

        }

        if (button_error == 0) // No such button error?
        {
            Serial.write (ch); // Print the received character

            if (sony == 1) // sony button, 0 = no, 1 = yes, set?
            {
                mySender.send (ary, sz, 40); // Pass the buffer, length, optionally frequency
                delayMicroseconds (10000); // delay 10 mS
            }

            if (nec == 1) // nec button, 0 = no, 1 = yes, set?
            {
                mySender.send (ary, sz, 38); // Pass the buffer, length, optionally frequency
                delayMicroseconds (19969); // delay 39.9375 mS
                delayMicroseconds (19969);
            }

            Serial.write (';'); // Terminate the print of the received character
        }

    }

}
