/*

------------------------------------------------------------------------------

A license is hereby granted to reproduce this software source code and
to create executable versions from this source code for personal,
non-commercial use.  The copyright notice included with the software
must be maintained in all copies produced.

THIS PROGRAM IS PROVIDED "AS IS". THE AUTHOR PROVIDES NO WARRANTIES
WHATSOEVER, EXPRESSED OR IMPLIED, INCLUDING WARRANTIES OF
MERCHANTABILITY, TITLE, OR FITNESS FOR ANY PARTICULAR PURPOSE.  THE
AUTHOR DOES NOT WARRANT THAT USE OF THIS PROGRAM DOES NOT INFRINGE THE
INTELLECTUAL PROPERTY RIGHTS OF ANY THIRD PARTY IN ANY COUNTRY.

Copyright (c) 1994-2018, John Conover, All Rights Reserved.

Comments and/or bug reports should be addressed to:

    john@email.johncon.com (John Conover)

------------------------------------------------------------------------------

xmp.timing.c is for changing the XMP timing values in a file derived
from the Arduino IRLib2 library, AnalysIR sketch, to the industry
"standard" timing values. The input file structure is a text file
consisting of records of white space delimited timing values, in
temporal order, with the '-' characters removed. The "standard" values
are printed to stdout.

From: http://www.hifi-remote.com/wiki/index.php/XMP

XMP uses one burst pair to encode numbers 0 to 15, with an ON duration
of 210 uS, and an OFF duration of 760 + (n * 136) uS where the value
of n is 0 to 15:

    A Mark is 210 uS

    A 0 Space is 760 uS
    A 1 Space is 896 uS
    A 2 Space is 1032 uS
    A 3 Space is 1168 uS
    A 4 Space is 1304 uS
    A 5 Space is 1440 uS
    A 6 Space is 1576 uS
    A 7 Space is 1712 uS
    A 8 Space is 1848 uS
    A 9 Space is 1984 uS
    A 10 Space is 2120 uS
    A 11 Space is 2256 uS
    A 12 Space is 2392 uS
    A 13 Space is 2528 uS
    A 14 Space is 2664 uS
    A 15 Space is 2800 uS

There is 136 uS between Space values, or +/- 68 uS:

    Bit   Minimum    Nominal    Maximum

    0     692        760        828
    1     828        896        964
    2     964        1032       1100
    3     1100       1168       1236
    4     1236       1304       1372
    5     1372       1440       1508
    6     1508       1576       1644
    7     1644       1712       1780
    8     1780       1848       1916
    9     1916       1984       2052
    10    2052       2120       2188
    11    2188       2256       2324
    12    2324       2392       2460
    13    2460       2528       2596
    14    2596       2664       2732
    15    2732       2800       2868

with, possibly, 1 subtracted from the maximum, and/or, 1 added to the
minimum.

$Revision: 0.0 $
$Date: 2018/01/18 19:36:00 $
$Id: xmp.timing.c,v 0.0 2018/01/18 19:36:00 john Exp $
$Log: xmp.timing.c,v $
Revision 0.0  2018/01/18 19:36:00  john
Initial version


*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <unistd.h>

#ifdef __STDC__

#include <float.h>

#endif

static char rcsid[] = "$Id: xmp.timing.c,v 0.0 2018/01/18 19:36:00 john Exp $"; /* program version */
static char copyright[] = "Copyright (c) 1994-2018, John Conover, All Rights Reserved"; /* the copyright banner */

#define BUFLEN BUFSIZ /* i/o buffer size */

#define TOKEN_SEPARATORS " \t\n\r\b\\,-{};" /* file record field separators */

#ifdef __STDC__

static const char *help_message[] = /* help message index array */

#else

static char *help_message[] = /* help message index array */

#endif

{
    "\n",
    "Change the XMP timing values in a file to industry standard values\n",
    "Usage: xmp.timing [-V] [-v] [filename]\n",
    "    -V, verbose output\n",
    "    -v, print the program's version information\n",
    "    filename, input filename\n"
};

#ifdef __STDC__

static const char *error_message[] = /* error message index array */

#else

static char *error_message[] = /* error message index array */

#endif

{
    "No error\n",
    "Error in program argument(s)\n",
    "Error opening file\n",
    "Error closing file\n"
};

#define NOERROR 0 /* error values, one for each index in the error message array */
#define EARGS 1
#define EOPEN 2
#define ECLOSE 3

#ifdef __STDC__

static void print_message (int retval); /* print any error messages */
static int strtoken (char *string, char *parse_array, char **parse, char *delim);

#else

static void print_message (); /* print any error messages */
static int strtoken ();

#endif

#ifdef __STDC__

int main (int argc, char *argv[])

#else

int main (argc, argv)
int argc;
char *argv[];

#endif

{
    char buffer[BUFLEN], /* i/o buffer */
         parsebuffer[BUFLEN], /* parsed i/o buffer */
         *token[BUFLEN / 2], /* reference to tokens in parsed i/o buffer */
         token_separators[] = TOKEN_SEPARATORS;

    int retval = NOERROR, /* return value, assume no error */
        fields, /* number of fields in a record */
        i, /* field counter of fields in a record */
        value, /* timing value of a field in a record */
        mark = 210, /* Mark ON duration of 210 uS */
        minvalue = 692, /* minimum timing value of a field in a record, excluding the Mark ON duration of 210 uS */
        maxvalue = 2868, /* maximum timing value of a field in a record */
        N, /* integer value of timing encode number of a field in a record, 0 <= n <= 15 */
        maxV, /* maximum possible timing value of a field in a record, integer value of a timing value */
        minVn, /* minimum possible timing value of a field in a record, next integer value of a timing value */
        nominalV, /* nominal possible timing value of a field in a record */
        verbose = 0, /* verbose output, 0 = no, 1 = yes */
        c; /* command line switch */

    double n; /* timing encode number of a field in a record, 0 <= n <= 15 */

    FILE *infile = stdin; /* reference to input file */

    while ((c = getopt (argc, argv, "Vv")) != EOF) /* for each command line switch */
    {

        switch (c) /* which switch? */
        {

            case 'V':

                verbose = 1; /* set the verbose output, 0 = no, 1 = yes */
                break;

            case 'v':

                (void) printf ("%s\n", rcsid); /* print the version */
                (void) printf ("%s\n", copyright); /* print the copyright */
                optind = argc; /* force argument error */
                retval = EARGS; /* assume not enough arguments */

            default: /* illegal switch? */

                optind = argc; /* force argument error */
                retval = EARGS; /* assume not enough arguments */
                break;
        }

    }

    if (retval == NOERROR) /* enough arguments? */
    {
        retval = EOPEN; /* assume error opening file */

        if ((infile = argc <= optind ? stdin : fopen (argv[optind], "r")) != (FILE *) 0) /* yes, open the input file */
        {
            retval = NOERROR; /* assume no error */

            while (fgets (buffer, BUFLEN, infile) != (char *) 0) /* read the records from the input file */
            {

                if ((fields = strtoken (buffer, parsebuffer, token, token_separators)) != 0) /* parse the record into fields, skip the record if there are no fields */
                {

                    if (token[0][0] != '#') /* if the first character of the first field is a '#' character, skip it */
                    {

                        for (i = 0; i < fields; i ++) /* for each field in the record */
                        {
                            value = atoi (token[i]); /* save the timing value of a field in a record */

                            if (value < minvalue) /* assume any timing value less than 692 is a Mark ON duration of 210 uS */
                            {
                                (void) printf ("%d\n", mark);
                            }

                            else if ((value > 0) && (value <= maxvalue)) /* timing value of a field in a record greater than zero and less than or equal to the maximum timing value of a field in a record */
                            {
                                n = (double) ((value - (double) 760) / (double) 136); /* save the timing encode number of a field in a record, 0 <= n <= 15 */

                                /*

                                Notice that the double n could be a
                                timing value between the integer value
                                of n, N, and, the integer value of N +
                                1.

                                If N is the nearest timing value:

                                    The lower bound will be 760 + (N *
                                    136) - 68.

                                    The upper bound will be 760 + (N *
                                    136) + 68.

                                If N + 1 is the nearest timing value:

                                    The lower bound will be 760 + ((N
                                    + 1) * 136) - 68.

                                    The upper bound will be 760 + ((N
                                    + 1) * 136) + 68.

                                Note that only the upper bound of N,
                                and, the lower bound of N + 1 have to
                                be tested to calculate the nearest
                                timing value.

                                */

                                N = (int) floor (n); /* save the integer value of timing encode number of a field in a record, 0 <= n <= 15 */
                                maxV =  760 + (N * 136) + 68; /* save the maximum possible timing value of a field in a record, integer value of a timing value */
                                minVn = 760 + ((N + 1) * 136) - 68; /* save the minimum possible timing value of a field in a record, next integer value of a timing value */

                                if (value < maxV) /* N'th standard timing value */
                                {
                                    nominalV = 760 + (N * 136); /* save the nominal possible timing value of a field in a record */

                                    if (verbose == 1) /* verbose output, 0 = no, 1 = yes, set? */
                                    {
                                        (void) printf ("%d, (%d, %d = %f%%)\n", nominalV, value, value - nominalV, (double) (((((double) value - (double) nominalV) / (double) nominalV) * (double) 100))); /* print the nominal timing value, the recorded value, the difference, and the percentage of the difference */
                                    }

                                    else
                                    {
                                        (void) printf ("%d\n", nominalV); /* print the nominal timing value */
                                    }

                                }

                                else if (value > minVn) /* N'th + 1 standard timing value */
                                {
                                    nominalV = 760 + ((N + 1) * 136); /* save the next nominal possible timing value of a field in a record */

                                    if (verbose == 1) /* verbose output, 0 = no, 1 = yes, set? */
                                    {
                                        (void) printf ("%d, (%d, %d = %f%%)\n", nominalV, value, value - nominalV, (double) (((((double) value - (double) nominalV) / (double) nominalV) * (double) 100))); /* print the nominal timing value, the recorded value, the difference, and the percentage of the difference */
                                    }

                                    else
                                    {
                                        (void) printf ("%d\n", nominalV);  /* print the nominal timing value */
                                    }

                                }

                                else
                                {
                                    (void) printf ("%s, %d, undeciphered timing value, *\n", token[i], value); /* something wrong, notification for manual intervention */
                                }

                            }

                            else
                            {

                                if (verbose == 1) /* verbose output, 0 = no, 1 = yes, set? */
                                {
                                    (void) printf ("%d, (%d, %d = %f%%)\n", 13800, value, value - 13800, (double) (((((double) value - (double) 13800) / (double) 13800) * (double) 100))); /* print the nominal timing value, the recorded value, the difference, and the percentage of the difference */
                                }

                                else
                                {
                                    (void) printf ("%s, %d\n", token[i], 13800); /* timing value out of bounds, may not be an error, perhaps a valid long duration Space, print notification for manual intervention */
                                }

                            }

                        }

                    }

                }

            }

            if (argc > optind) /* using stdin as input? */
            {

                if (fclose (infile) == EOF) /* no, close the input file */
                {
                    retval = ECLOSE; /* error closing file */
                }

            }

        }

    }

    print_message (retval); /* print any error messages */
    exit (retval); /* exit with the error value */

#ifdef LINT

    return (0); /* for lint formalities */

#endif

}

/*

Print any error messages.

static void print_message (int retval);

I) Data structures:

    A) The help_message array is an array of character stings, one per
       line to be printed for help.

    B) The error_message array is an array of character strings, one
       line per error; the error_message array is implicitly addressed
       by the value integer retval, which specifies the error to be
       printed.

II) Function execution:

    A) Depending on the value of the integer retval:

        1) If retval is zero, print nothing-normal/successful program
           exit.

        2) Else if retval is unity, print help.

        3) Else retval is not zero or unity, it is an error code, print
           the corresponding error message.

Returns nothing.

*/

#ifdef __STDC__

static void print_message (int retval)

#else

static void print_message (retval)
int retval;

#endif

{
    size_t help_ctr; /* help_message line counter */

    switch (retval) /* which return value */
    {

        case 0: /* program ended without errors, print nothing */

            break;

        case 1: /* program ended with a request for help, or argument error, print help */

            for (help_ctr = 0; help_ctr < (sizeof (help_message) / sizeof (char *)); help_ctr ++) /* for each line of help */
            {
                (void) printf ("%s", help_message[help_ctr]); /* print the line */
            }

            break;

        default: /* an error that was not a request for help, print the error */

            (void) fprintf (stderr, "%s", error_message[retval]);
            break;

    }

}

/*

parse a record based on sequential delimiters

int strtoken (char *string, char *parse_array, char **parse, char *delim);

parse a character array, string, into an array, parse_array, using
consecutive characters from delim as field delimiters, point the
character pointers, token, to the beginning of each field, return the
number of fields parsed

*/

#ifdef __STDC__

static int strtoken (char *string, char *parse_array, char **parse, char *delim)

#else

static int strtoken (string, parse_array, parse, delim)
char *string,
*parse_array,
**parse,
*delim;

#endif

{
    int tokens = 0;

    (void) strcpy (parse_array, string); /* copy the string */

    parse[tokens] = strtok (parse_array, delim); /* get the 1st field */

    while (parse[tokens] != 0) /* get the remaining fields */
    {
        parse[++ tokens] = strtok ((char *) 0, delim);
    }

    return (tokens); /* return the number of tokens parsed */
}
